@@ -60,8 +60,12 @@ use std::thread;
 
 #[cfg(unix)]
 use async_io::Async;
+#[cfg(unix)]
+use std::os::unix::io::AsRawFd;
+
 #[cfg(windows)]
 use blocking::Unblock;
+
 use event_listener::Event;
 use futures_lite::{future, io, prelude::*};
 use once_cell::sync::Lazy;
@@ -77,10 +81,11 @@ pub mod windows;
 /// An event delivered every time the SIGCHLD signal occurs.
 static SIGCHLD: Event = Event::new();
 
-/// A guard that pushes abandoned child processes into the zombie list.
+/// A guard that can kill child processes, or push them into the zombie list.
 struct ChildGuard {
     inner: Option<std::process::Child>,
     reap_on_drop: bool,
+    kill_on_drop: bool,
 }
 
 impl ChildGuard {
@@ -119,9 +124,6 @@ pub struct Child {
 
     /// The inner child process handle.
     child: Arc<Mutex<ChildGuard>>,
-
-    /// Whether to kill the process on drop.
-    kill_on_drop: bool,
 }
 
 impl Child {
@@ -186,9 +188,11 @@ impl Child {
                 }
 
             } else if #[cfg(unix)] {
-                static SIGNALS: Lazy<signal_hook::iterator::Signals> = Lazy::new(|| {
-                    signal_hook::iterator::Signals::new(&[signal_hook::SIGCHLD])
-                        .expect("cannot set signal handler for SIGCHLD")
+                static SIGNALS: Lazy<Mutex<signal_hook::iterator::Signals>> = Lazy::new(|| {
+                    Mutex::new(
+                        signal_hook::iterator::Signals::new(&[signal_hook::consts::SIGCHLD])
+                            .expect("cannot set signal handler for SIGCHLD"),
+                    )
                 });
 
                 // Make sure the signal handler is registered before interacting with the process.
@@ -196,7 +200,7 @@ impl Child {
 
                 // Waits for the next SIGCHLD signal.
                 fn wait_sigchld() {
-                    SIGNALS.forever().next();
+                    SIGNALS.lock().unwrap().forever().next();
                 }
 
                 // Wraps a sync I/O type into an async I/O type.
@@ -241,6 +245,9 @@ impl Child {
         // When the last reference to the child process is dropped, push it into the zombie list.
         impl Drop for ChildGuard {
             fn drop(&mut self) {
+                if self.kill_on_drop {
+                    self.get_mut().kill().ok();
+                }
                 if self.reap_on_drop {
                     let mut zombies = ZOMBIES.lock().unwrap();
                     if let Ok(None) = self.get_mut().try_wait() {
@@ -257,8 +264,8 @@ impl Child {
             child: Arc::new(Mutex::new(ChildGuard {
                 inner: Some(child),
                 reap_on_drop: cmd.reap_on_drop,
+                kill_on_drop: cmd.kill_on_drop,
             })),
-            kill_on_drop: cmd.kill_on_drop,
         })
     }
 
@@ -420,14 +427,6 @@ impl Child {
     }
 }
 
-impl Drop for Child {
-    fn drop(&mut self) {
-        if self.kill_on_drop {
-            self.kill().ok();
-        }
-    }
-}
-
 impl fmt::Debug for Child {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         f.debug_struct("Child")
@@ -448,6 +447,38 @@ pub struct ChildStdin(
     #[cfg(unix)] Async<std::process::ChildStdin>,
 );
 
+impl ChildStdin {
+    /// Convert async_process::ChildStdin into std::process::Stdio.
+    ///
+    /// You can use it to associate to the next process.
+    ///
+    /// # Examples
+    ///
+    /// ```no_run
+    /// # futures_lite::future::block_on(async {
+    /// use async_process::Command;
+    /// use std::process::Stdio;
+    ///
+    /// let mut ls_child = Command::new("ls").stdin(Stdio::piped()).spawn()?;
+    /// let stdio:Stdio = ls_child.stdin.take().unwrap().into_stdio().await?;
+    ///
+    /// let mut echo_child = Command::new("echo").arg("./").stdout(stdio).spawn()?;
+    ///
+    /// # std::io::Result::Ok(()) });
+    /// ```
+    pub async fn into_stdio(self) -> io::Result<std::process::Stdio> {
+        cfg_if::cfg_if! {
+            if #[cfg(windows)] {
+                Ok(self.0.into_inner().await.into())
+            } else if #[cfg(unix)] {
+                let child_stdin = self.0.into_inner()?;
+                blocking_fd(child_stdin.as_raw_fd())?;
+                Ok(child_stdin.into())
+            }
+        }
+    }
+}
+
 impl io::AsyncWrite for ChildStdin {
     fn poll_write(
         mut self: Pin<&mut Self>,
@@ -475,6 +506,41 @@ pub struct ChildStdout(
     #[cfg(unix)] Async<std::process::ChildStdout>,
 );
 
+impl ChildStdout {
+    /// Convert async_process::ChildStdout into std::process::Stdio.
+    ///
+    /// You can use it to associate to the next process.
+    ///
+    /// # Examples
+    ///
+    /// ```no_run
+    /// # futures_lite::future::block_on(async {
+    /// use async_process::Command;
+    /// use std::process::Stdio;
+    /// use std::io::Read;
+    /// use futures_lite::AsyncReadExt;
+    ///
+    /// let mut ls_child = Command::new("ls").stdout(Stdio::piped()).spawn()?;
+    /// let stdio:Stdio = ls_child.stdout.take().unwrap().into_stdio().await?;
+    ///
+    /// let mut echo_child = Command::new("echo").stdin(stdio).stdout(Stdio::piped()).spawn()?;
+    /// let mut buf = vec![];
+    /// echo_child.stdout.take().unwrap().read(&mut buf).await;
+    /// # std::io::Result::Ok(()) });
+    /// ```
+    pub async fn into_stdio(self) -> io::Result<std::process::Stdio> {
+        cfg_if::cfg_if! {
+            if #[cfg(windows)] {
+                Ok(self.0.into_inner().await.into())
+            } else if #[cfg(unix)] {
+                let child_stdout = self.0.into_inner()?;
+                blocking_fd(child_stdout.as_raw_fd())?;
+                Ok(child_stdout.into())
+            }
+        }
+    }
+}
+
 impl io::AsyncRead for ChildStdout {
     fn poll_read(
         mut self: Pin<&mut Self>,
@@ -494,6 +560,37 @@ pub struct ChildStderr(
     #[cfg(unix)] Async<std::process::ChildStderr>,
 );
 
+impl ChildStderr {
+    /// Convert async_process::ChildStderr into std::process::Stdio.
+    ///
+    /// You can use it to associate to the next process.
+    ///
+    /// # Examples
+    ///
+    /// ```no_run
+    /// # futures_lite::future::block_on(async {
+    /// use async_process::Command;
+    /// use std::process::Stdio;
+    ///
+    /// let mut ls_child = Command::new("ls").arg("x").stderr(Stdio::piped()).spawn()?;
+    /// let stdio:Stdio = ls_child.stderr.take().unwrap().into_stdio().await?;
+    ///
+    /// let mut echo_child = Command::new("echo").stdin(stdio).spawn()?;
+    /// # std::io::Result::Ok(()) });
+    /// ```
+    pub async fn into_stdio(self) -> io::Result<std::process::Stdio> {
+        cfg_if::cfg_if! {
+            if #[cfg(windows)] {
+                Ok(self.0.into_inner().await.into())
+            } else if #[cfg(unix)] {
+                let child_stderr = self.0.into_inner()?;
+                blocking_fd(child_stderr.as_raw_fd())?;
+                Ok(child_stderr.into())
+            }
+        }
+    }
+}
+
 impl io::AsyncRead for ChildStderr {
     fn poll_read(
         mut self: Pin<&mut Self>,
@@ -782,9 +879,9 @@ impl Command {
     /// ```
     pub fn spawn(&mut self) -> io::Result<Child> {
         let (stdin, stdout, stderr) = (self.stdin.take(), self.stdout.take(), self.stderr.take());
-        self.inner.stdin(stdin.unwrap_or(Stdio::inherit()));
-        self.inner.stdout(stdout.unwrap_or(Stdio::inherit()));
-        self.inner.stderr(stderr.unwrap_or(Stdio::inherit()));
+        self.inner.stdin(stdin.unwrap_or_else(Stdio::inherit));
+        self.inner.stdout(stdout.unwrap_or_else(Stdio::inherit));
+        self.inner.stderr(stderr.unwrap_or_else(Stdio::inherit));
 
         Child::new(self)
     }
@@ -826,16 +923,79 @@ impl Command {
     /// # futures_lite::future::block_on(async {
     /// use async_process::Command;
     ///
-    /// let child = Command::new("ls").spawn()?;
+    /// let output = Command::new("cat")
+    ///     .arg("a.txt")
+    ///     .output()
+    ///     .await?;
     /// # std::io::Result::Ok(()) });
     /// ```
     pub fn output(&mut self) -> impl Future<Output = io::Result<Output>> {
         let (stdin, stdout, stderr) = (self.stdin.take(), self.stdout.take(), self.stderr.take());
-        self.inner.stdin(stdin.unwrap_or(Stdio::null()));
-        self.inner.stdout(stdout.unwrap_or(Stdio::piped()));
-        self.inner.stderr(stderr.unwrap_or(Stdio::piped()));
+        self.inner.stdin(stdin.unwrap_or_else(Stdio::null));
+        self.inner.stdout(stdout.unwrap_or_else(Stdio::piped));
+        self.inner.stderr(stderr.unwrap_or_else(Stdio::piped));
 
         let child = Child::new(self);
         async { child?.output().await }
     }
 }
+
+/// Moves `Fd` out of non-blocking mode.
+#[cfg(unix)]
+fn blocking_fd(fd: std::os::unix::io::RawFd) -> io::Result<()> {
+    // Helper macro to execute a system call that returns an `io::Result`.
+    macro_rules! syscall {
+        ($fn:ident ( $($arg:expr),* $(,)? ) ) => {{
+            let res = unsafe { libc::$fn($($arg, )*) };
+            if res == -1 {
+                return Err(std::io::Error::last_os_error());
+            } else {
+                res
+            }
+        }};
+    }
+
+    let res = syscall!(fcntl(fd, libc::F_GETFL));
+    syscall!(fcntl(fd, libc::F_SETFL, res & !libc::O_NONBLOCK));
+
+    Ok(())
+}
+
+#[cfg(unix)]
+mod test {
+
+    #[test]
+    fn test_into_inner() {
+        futures_lite::future::block_on(async {
+            use crate::Command;
+
+            use std::io::Result;
+            use std::process::Stdio;
+            use std::str::from_utf8;
+
+            use futures_lite::AsyncReadExt;
+
+            let mut ls_child = Command::new("cat")
+                .arg("Cargo.toml")
+                .stdout(Stdio::piped())
+                .spawn()?;
+
+            let stdio: Stdio = ls_child.stdout.take().unwrap().into_stdio().await?;
+
+            let mut echo_child = Command::new("grep")
+                .arg("async")
+                .stdin(stdio)
+                .stdout(Stdio::piped())
+                .spawn()?;
+
+            let mut buf = vec![];
+            let mut stdout = echo_child.stdout.take().unwrap();
+
+            stdout.read_to_end(&mut buf).await?;
+            dbg!(from_utf8(&buf).unwrap_or(""));
+
+            Result::Ok(())
+        })
+        .unwrap();
+    }
+}
